/* IluJava_IluDebug.c */
/* Chris Jacobi, April 18, 1997 3:31 pm PDT */

/*
 * Copyright (c) 1996 Xerox Corporation.  All Rights Reserved.  
 * Unlimited use, reproduction, and distribution of this software is
 * permitted.  Any copy of this software must include both the above
 * copyright notice of Xerox Corporation and this paragraph.  Any
 * distribution of this software must comply with all applicable United
 * States export control laws.  This software is made available AS IS,
 * and XEROX CORPORATION DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
 * INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE, AND NOTWITHSTANDING ANY OTHER
 * PROVISION CONTAINED HEREIN, ANY LIABILITY FOR DAMAGES RESULTING FROM
 * THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 * CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 * XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
 
/* $Id: IluJava_IluDebug.c,v 1.12 1997/04/18 23:06:21 jacobi Exp $ */
 
/*
 * Native impl for IluDebug.java, IluDebugStream.java, IluDebugHooks.java
 * including a few native utilities...
 */


#include "IluJava.h"
#include <javaString.h>
#include "xerox_ilu_IluDebugHooks.h"
#include "xerox_ilu_IluDebug.h"
#include "xerox_ilu_IluDebugStream.h"
#include "xerox_ilu_IluRT0.h"
#include "xerox_ilu_IluOInt.h"
#include "xerox_ilu_IluClassRep.h"
#include "xerox_ilu_IluServer.h"

#include "IluJava_JTypes.h"
#define JIluDebug Hxerox_ilu_IluDebug*
#define JIluDebugHooks Hxerox_ilu_IluDebugHooks*


static ilu_boolean _ilujava_langRegistered = FALSE;
static ilu_LanguageIndex _ilujava_langId = 999;


ilu_LanguageIndex _ilujava_LangIdx(void)
{
    if (! _ilujava_langRegistered) {
        _ilujava_langId = ilu_RegisterLanguage("Java");
        _ilujava_langRegistered = ilu_TRUE;
    }
    return _ilujava_langId;
}


EXPORTLIMITEDVAR char * _ilujava_notUsed1;
EXPORTLIMITEDVAR char * _ilujava_notUsed2;
/* Used to fool C compiler into believing a pointer is still in
 * use:  Assign to this to preven C compiler from discarding 
 * pointer -> so Java garbage collector will find the reference on the 
 * stack...
 */


EXPORTLIMITED void
_ilujava_BUG(char * f)
{
    ilu_DebugPrintf("$** Ilu_Java BUG: %s \n", f);
    if (_ilujava_iluGenericFlag > 1) {
        _ilujava_PANIC(f);
    }
    SignalError(EE(), "xerox/ilu/IluSomeSystemException", "**_ilujava_BUG");
}


EXPORTLIMITED void
_ilujava_PANIC(char * f)
{
    ilu_DebugPrintf("$** Ilu_Java PANIC: %s \n", f);
    _ilu_Assert(0, f);
}


/* immutable instances from set up */
static JIluDebug      _ilujava_jjIluDebugTheInst = 0;
static JIluDebugHooks _ilujava_jjIluDebugHooksTheInst = 0;

/* dynamicly registered instances */
static JIluDebugHooks _ilujava_jjAssertionFailure = 0;
static JIluDebugHooks _ilujava_jjCheckFailure     = 0;
static JIluDebugHooks _ilujava_jjDebugHook        = 0;  


INTERNALONLY void 
_ilujava_RaiseDebugHook(ilu_ErrorType et, const char *file, int line)
/* Register this with Ilu;
 * It is ok here to do no action if we are not debugging
 */
{
    JIluDebugHooks jjdh = _ilujava_jjDebugHook;
    if (jjdh) {
        struct Hjava_lang_String * js = 0;
        long li = line;
        long let = (long) et;
        if (file) {
            /*javaString.h*/ 
            js = makeJavaString((char *) file, strlen(file));
        }
        execute_java_dynamic_method(EE(),
            (struct Hjava_lang_Object *) jjdh,
            "reportDebugHook", 
            "(Ljava/lang/String;II)V",
            js, 
            li,
            let);
    }
}


INTERNALONLY void 
_ilujava_AssertionFailConsumer(const char *file, int line)
/* Register this with Ilu;
 * Always do something
 */
{
    JIluDebugHooks jjaf = _ilujava_jjAssertionFailure;
    if (jjaf) {
        struct Hjava_lang_String * js = 0;
        long li = line;
        if (file) {
            /*javaString.h*/ 
            js = makeJavaString((char *) file, strlen(file));
        }
        execute_java_dynamic_method(EE(),
            (struct Hjava_lang_Object *) jjaf,
            "reportAssertionFailure", 
            "(Ljava/lang/String;I)V",
            js, 
            li);
    } else {
        _ilujava_BUG("ASSERTION FAILURE");
    }
}


INTERNALONLY void 
_ilujava_CheckFailureConsumer(const char *file, int line)
/* Register this with Ilu;
 * Always do something
 */
{
    JIluDebugHooks jjdh = _ilujava_jjCheckFailure;
    if (jjdh) {
        struct Hjava_lang_String * js = 0;
        long li = line;
        if (file) {
            /*javaString.h*/ 
            js = makeJavaString((char *) file, strlen(file));
        }
        execute_java_dynamic_method(EE(),
            (struct Hjava_lang_Object *) jjdh,
            "reportCheckFailure", 
            "(Ljava/lang/String;I)V",
            js, 
            li);
    } else {
        _ilujava_BUG("CONSISTENCY CHECK");
    }
}


EXPORTJAVA void 
xerox_ilu_IluDebug_registerTheInst(JIluDebug jjd)
/* Called once at initialization */
{
    _ilujava_jjIluDebugTheInst = jjd;
}


EXPORTJAVA void 
xerox_ilu_IluDebugHooks_registerTheInst(JIluDebugHooks jjdh)
/* Called once at initialization */
{
    _ilujava_jjIluDebugHooksTheInst = jjdh;
    /* set a few defaults */
    if (_ilujava_jjAssertionFailure==0) _ilujava_jjAssertionFailure = jjdh;
    if (_ilujava_jjCheckFailure==0)     _ilujava_jjCheckFailure     = jjdh;
    /* but don't automatically set jjDebugHook */
}


EXPORTJAVA void 
xerox_ilu_IluDebugHooks_defineDebugHook(
    JIluDebugHooks unused, 
    JIluDebugHooks jdbh
    )
/* Java client registers an IluDebugHooks instance (or subclass).
 * It is ok to be not debugging...
 */
{
    _ilujava_jjDebugHook = jdbh;
    if (jdbh) {
         ilu_SetRaiseDebugHook(&_ilujava_RaiseDebugHook);
    } else {
         ilu_SetRaiseDebugHook(0);
    }
}


EXPORTJAVA void
xerox_ilu_IluDebugHooks_defineAssertionFailure(
    JIluDebugHooks unused, 
    JIluDebugHooks jjaf
    )
/* Java client registers an IluDebugHooks instance (or subclass).
 * It makes no sense to do nothing.
 */
{
    if (jjaf==0) jjaf = _ilujava_jjIluDebugHooksTheInst;
    _ilujava_jjAssertionFailure = jjaf;
    ilu_SetAssertionFailConsumer(&_ilujava_AssertionFailConsumer);
}
        

EXPORTJAVA void 
xerox_ilu_IluDebugHooks_defineCheckFailure(
    JIluDebugHooks unused, 
    JIluDebugHooks jjcf
    )
/* Java client registers an IluDebugHooks instance (or subclass).
 * It makes no sense to do nothing.
 */
{
    if (jjcf==0) jjcf = _ilujava_jjIluDebugHooksTheInst;
    _ilujava_jjCheckFailure = jjcf;
    ilu_SetCheckFailureConsumer(&_ilujava_CheckFailureConsumer);
}


EXPORTLIMITEDVAR int _ilujava_iluGenericFlag = 99;
EXPORTLIMITEDVAR int _ilujava_iluSpare1Flag    = 99;
EXPORTLIMITEDVAR int _ilujava_iluSpare2Flag    = 99;
EXPORTLIMITEDVAR int _ilujava_objectsFlag      = 99;
EXPORTLIMITEDVAR int _ilujava_pickleFlag       = 99;
EXPORTLIMITEDVAR int _ilujava_callsFlag        = 99;
EXPORTLIMITEDVAR int _ilujava_definitionsFlag  = 99;
EXPORTLIMITEDVAR int _ilujava_pctFlag          = 99;
EXPORTLIMITEDVAR int _ilujava_traceFlag        = 99;
EXPORTLIMITEDVAR int _ilujava_gcFlag           = 99;
EXPORTLIMITEDVAR int _ilujava_lockFlag         =  0;
EXPORTLIMITEDVAR int _ilujava_alarmFlag        =  0;


EXPORTJAVA void
xerox_ilu_IluDebug_reportFlags(
    JIluDebug unused, 
    Jint iluGenericFlag,
    Jint iluSpare1Flag,
    Jint iluSpare2Flag,
    Jint objectsFlag,
    Jint pickleFlag,
    Jint callsFlag,
    Jint definitionsFlag,
    Jint pctFlag,
    Jint gcFlag,
    Jint lockFlag,
    Jint alarmFlag
    )
{
    _ilujava_iluGenericFlag   = iluGenericFlag;
    _ilujava_iluSpare1Flag      = iluSpare1Flag;
    _ilujava_iluSpare2Flag      = iluSpare2Flag;
    _ilujava_objectsFlag        = objectsFlag;
    _ilujava_pickleFlag         = pickleFlag;
    _ilujava_callsFlag          = callsFlag;
    _ilujava_definitionsFlag    = definitionsFlag;
    _ilujava_pctFlag            = pctFlag;
    _ilujava_gcFlag             = gcFlag;
    _ilujava_lockFlag           = lockFlag;
    _ilujava_alarmFlag          = alarmFlag;
}



EXPORTJAVA void
xerox_ilu_IluDebugStream_debugWrite1(
    struct Hxerox_ilu_IluDebugStream * unused, 
    Jint b
    )
{
    ilu_DebugPrintf("%c", b);
}


EXPORTJAVA void
xerox_ilu_IluDebugStream_debugWriteN(
    struct Hxerox_ilu_IluDebugStream * unused, 
    HArrayOfByte * jjb,
    Jint off,
    Jint len
    )
{
    Jint sz;
    char * ptr;
    if (jjb) {
        sz = obj_length(jjb);
        ptr = (char *) unhand(jjb)->body;
        while (len>0 & off<sz) {
            ilu_DebugPrintf("%c", ptr[off]);
            off = off+1;
            len = len-1;
        }
    }
}


EXPORTLIMITED char * 
_ilujava_captureJavaStack()
{
    JIluDebug jjDebInst = _ilujava_jjIluDebugTheInst;
    if (jjDebInst) {
        struct Hjava_lang_String * js;
        js = (struct Hjava_lang_String *) execute_java_dynamic_method(0,
            (struct Hjava_lang_Object *) jjDebInst,
            "dynamicCaptureStackTrace", 
            "()Ljava/lang/String;"
            );
        if (js) {
            return /*javaString*/ allocCString(js);
        }
    } 
    return 0;
}


EXPORTLIMITED int 
_ilujava_getIntProp(char * key, int deflt)
/* Read a java property value */
{
    int val = deflt;
    JIluDebug jjDebInst = _ilujava_jjIluDebugTheInst;
    if (jjDebInst != 0 AND key != 0) {
        long ldeflt = deflt;
        struct Hjava_lang_String * js = makeJavaString(key, strlen(key));
        val = (int) execute_java_dynamic_method(EE(),
            (struct Hjava_lang_Object *) jjDebInst,
            "dynamicGetIntProp", 
            "(Ljava/lang/String;I)I",
            js,
            ldeflt
            );
    } 
    return val;
}


/* end */
