/* IluJava_JOps.c */
/* Chris Jacobi, April 29, 1997 6:39 pm PDT */

/*
 * Copyright (c) 1996 Xerox Corporation.  All Rights Reserved.  
 * Unlimited use, reproduction, and distribution of this software is
 * permitted.  Any copy of this software must include both the above
 * copyright notice of Xerox Corporation and this paragraph.  Any
 * distribution of this software must comply with all applicable United
 * States export control laws.  This software is made available AS IS,
 * and XEROX CORPORATION DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
 * INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE, AND NOTWITHSTANDING ANY OTHER
 * PROVISION CONTAINED HEREIN, ANY LIABILITY FOR DAMAGES RESULTING FROM
 * THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 * CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 * XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
 
/* $Id: IluJava_JOps.c,v 1.7 1997/04/30 01:39:45 jacobi Exp $ */

/*
 * Handy java access operations.
 * Implementation makes disgusting use of java implementation functionality
 */

#include <StubPreamble.h>
#include <interpreter.h>
#include <javaString.h>
#include "IluJava_JOps.h"
#include <typecodes.h>


extern HArrayOfByte* 
IluJava_JAoB_from8(char* data, unsigned len) 
    /* Allocates and initializes a Java array of byte.
     * data: initial data interpreted as 8 bit bytes 
     * len: length of data (or array) in 8 bit units
     * Caller owns data. 
     */
{
    HArrayOfByte* jja;
    jja = (HArrayOfByte*) ArrayAlloc(T_BYTE, len);
    if (jja == 0) {
        SignalError(EE(), "java/lang/OutOfMemoryError", "IluJava_JAoB_from8");
    } else if (len) {
        memcpy((char*) unhand(jja)->body, (char*) data, len);
    }
    return jja;
}

static void
ilujava_copy8to16(char* src, unicode* dst, unsigned len)
/* strictly converts and copies len units; no
 * fancy terminator charactors.
 */
{
    while (len) {
        *dst++ = *src++;
        len--;
    }
}
     

extern HArrayOfChar * 
IluJava_JAoC_from8(char * data, unsigned len) 
    /* Allocates and initializes a Java array of char
     * data: initial data interpreted as 8 bit characters 
     * len: length of data in 8 bit units
     * Caller owns data 
     */
{
    HArrayOfChar* jja;
    jja = (HArrayOfChar*) ArrayAlloc(T_CHAR, len);
    if (jja == 0) {
        SignalError(EE(), "java/lang/OutOfMemoryError", "IluJava_JAoC_from8");
    } else if (len) {
        ilujava_copy8to16((char*) data, (unicode*) unhand(jja)->body, len);
    }
    return jja;
}


extern HArrayOfChar * 
IluJava_JAoC_from16(char * data, unsigned len) 
    /* Allocates and initializes a Java array of char.
     * data: initial data interpreted as 16 bit characters 
     * len: length of data in 16 bit units
     * Caller owns data 
     */
{
    HArrayOfChar* jja;
    jja = (HArrayOfChar*) ArrayAlloc(T_CHAR, len);
    if (jja == 0) {
        SignalError(EE(), "java/lang/OutOfMemoryError", "IluJava_JAoC_from16");
    } else if (len) {
        memcpy((char*) unhand(jja)->body, (char*) data, len*2);
    }
    return jja;
}


extern void 
IluJava_JString_toC16(Hjava_lang_String * jjs, char * buffer, unsigned len) 
    /* Copies 16 bit characters from a java String.
     * buffer: "array" of 16 bit character units 
     * len: length of buffer in 16 bit units
     * Caller owns buffer  
     */
{
    /* javaString.h */
    javaString2unicode(jjs, (unicode *) buffer, len);
}
  
     
extern void 
IluJava_JString_toC8(Hjava_lang_String * jjs, char * buffer, unsigned len) 
    /* Copies characters from a java String.
     * Characters must fit in 8 bit units. 
     * buffer: array" of 8 bit character units 
     * len: length of buffer in 8 bit units
     * Caller owns buffer 
     */
{	
    /* javaString.h */
    javaString2CString(jjs, buffer, len);
}


extern void 
IluJava_JAoC_toC16(HArrayOfChar * jja, char * buffer, unsigned cnt) 
    /* Copies java array of char into a C 16-bit buffer.
     * buffer: "array" of 16 bit character units; must be long 
     * enough to receive all cnt chars plus terminating 0C unit. 
     * cnt: Number of 8 bit units to copy  (Must be >= 1 )
     * Caller owns buffer  
     * Ugly but checked November 26, 1996 11:07:36 pm PST 
     */
{
    /* compare to string.c::javaString2unicode */
    memcpy(buffer, unhand(jja)->body, cnt*2);
    buffer[cnt*2] = 0;
    buffer[cnt*2+1] = 0;
}


static void
ilujava_copy16to8(unicode *src, char *dst, unsigned len)
/* strictly converts and copies len units; no
 * fancy terminator charactors.
 */
{
    while (len) {
        *dst++ = *src++;
        len--;
    }
}
     
extern void 
IluJava_JAoC_toC8(HArrayOfChar * jja, char * buffer, unsigned cnt) 
    /* Copies java array of char into a C string buffer.
     * Restricted to 8 bit units of the java chars. 
     * buffer: string buffer; must be long enough to receive 
     * all cnt chars plus terminating 0C char.
     * cnt: Number of 8 bit units to copy  (Must be >= 1 )
     * Caller owns buffer; 
     */
{
    /* compare to string.c::javaString2CString */
    ilujava_copy16to8(unhand(jja)->body, buffer, cnt);
    buffer[cnt] = 0;
}	

/* end */




