/* context.c */
/* Chris Jacobi, April 15, 1997 1:18 pm PDT */

/*
 * Copyright (c) 1991-1996 Xerox Corporation.  All Rights Reserved.
 * Unlimited use, reproduction, and distribution of this software is permitted.
 * Any copy of this software must include both the above copyright notice of
 * Xerox Corporation and this paragraph.  Any distribution of this software
 * must comply with all applicable United States export control laws.  This
 * software is made available AS IS, and XEROX CORPORATION DISCLAIMS ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND
 * NOTWITHSTANDING ANY OTHER PROVISION CONTAINED HEREIN, ANY LIABILITY FOR
 * DAMAGES RESULTING FROM THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED,
 * WHETHER ARISING IN CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT
 * LIABILITY, EVEN IF XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGES.
 */
 
/* $Id: context.c,v 1.10 1997/04/15 20:47:13 jacobi Exp $ */


#include <stdio.h>
#include <string.h>
#include "iluptype.h" 
#include "shared.h"
#include "context.h"
#include "stubops.h"


PRIVATE IHandle defaultContext = 0;
PRIVATE list ifList = 0;


PRIVATE boolean ihMatch(IHandle ctx, char* b)
{
    return (strcmp(ctx->islIfName, b) == 0);
}


PUBLIC IHandle newContext()
{
    IHandle c = (IHandle) iluparser_Malloc(sizeof(struct context_s));
    c->islIfName = "";
    c->prefix = 0;
    c->dirName = 0;
    c->flatDir = 0;
    c->visiHlp = 0;
    c->sunHld = 0;
    c->noServer = 0;
    c->useDel = 0;
    c->forbidden = 0;
    c->stubSuffix = 0;
    c->refSuffix = 0;
    c->holderSuffix = 0;
    c->helperSuffix = 0;
    c->delClassSuffix = 0;
    c->exceptSuffix = 0;
    c->loaderClass_Name = 0;
    c->stuffToLoad = new_list();
    return c;
}


PRIVATE IHandle ihCopy(IHandle ctx)
/* BUT NOT THE stuffToLoad FIELD! */
{
    IHandle c = newContext();
    list x = c->stuffToLoad;
    *c = *ctx;
    c->stuffToLoad = x;
    return c;
}

PUBLIC void setDefaultContext(IHandle ctx)
{
    defaultContext = ihCopy(ctx);
}


PRIVATE void init()
{
    struct context_s ctx;
    if (ifList==0) ifList = iluparser_new_list();
    if (defaultContext==0) {
        ctx.prefix = "stubs";
        setDefaultContext(&ctx);
    }
}


PUBLIC IHandle getContextN(char * name)
{
    IHandle found;
    if (name==0) name = "";
    found = (IHandle) list_find(ifList, (iluparser_FindProc) ihMatch, name);
    if (found == 0){
        if (ifList==0) init();
        found = ihCopy(defaultContext);
        found->islIfName = name;
        list_insert(ifList, found);
    }
    return found;
}


PUBLIC void setContextN(char * name, IHandle ctx)
{
    IHandle found;
    if (name==0) name = "";
    if (strcmp(ctx->islIfName, name) != 0) {
        fatal("bad use of setContextN");
    }
    found = (IHandle) list_find(ifList, (iluparser_FindProc) ihMatch, name);
    if (found) {
        if (found == ctx) return;
        list_remove(ifList, found);
    }
    if (ifList==0) init();
    list_insert(ifList, ctx);
}


PUBLIC IHandle getContext(Interface ifc)
{
    char * name = interface_name(ifc);
    return getContextN(name);
}


PUBLIC void setContext(Interface ifc, IHandle ctx)
{
    char * name = interface_name(ifc);
    if (strcmp(ctx->islIfName, name) != 0) {
        fatal("bad use of setContext");
    }
    setContextN(name, ctx);
}


PUBLIC IHandle getContextT(Type t)
{
    return getContext(type_interface(t));
}

    
