/* genex.c */
/* Chris Jacobi, April 20, 1997 2:16 pm PDT */

/*
 * Copyright (c) 1991-1996 Xerox Corporation.  All Rights Reserved.
 * Unlimited use, reproduction, and distribution of this software is permitted.
 * Any copy of this software must include both the above copyright notice of
 * Xerox Corporation and this paragraph.  Any distribution of this software
 * must comply with all applicable United States export control laws.  This
 * software is made available AS IS, and XEROX CORPORATION DISCLAIMS ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND
 * NOTWITHSTANDING ANY OTHER PROVISION CONTAINED HEREIN, ANY LIABILITY FOR
 * DAMAGES RESULTING FROM THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED,
 * WHETHER ARISING IN CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT
 * LIABILITY, EVEN IF XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGES.
 */

/* $Id: genex.c,v 1.10 1997/04/20 21:22:01 jacobi Exp $ */


#include <stdio.h>
#include <string.h>

#include "iluptype.h"
#include "shared.h"
#include "stubops.h"
#include "name.h"
#include "util.h"
#include "context.h"
#include "genex.h"
#include "genrecord.h"
#include "io.h"


PRIVATE char*
methodNameOfWriteProc(Exception e)
{
    char* shortname = exceptionName(e);
    return cat3("_", shortname, "_marshall"); 
}


PUBLIC char*
fullNameOfWriteProc(Exception e)
{
    if (e->type) {
        return cat5(
            packageName(e->interface),	/*Package*/
            ".",
            exceptionName(e),         	/*Class*/
            ".",
            methodNameOfWriteProc(e) 	/*Method*/
            ); 
    } else {
        return "xerox.ilu.IluUserException.writeException";
    }
}

PRIVATE boolean
isIDLExceptionRecord(Type t)
{
    if (rec_is_a(t)) {
        char* idlTypeName = type_name(t);
        return isPrefixOf(idlExceptionPrefix, idlTypeName); 
    }
    return FALSE;
}


PUBLIC void
map_exception(Exception e, void* x)
{
    boolean idlHack = FALSE;
    char* shortname;
    char* recField = "value";
    
    if (e->import != 0) {
        return;
    }
    shortname = exceptionName(e);
    if (e->type) {
        idlHack = isIDLExceptionRecord(e->type);
        if (idlHack) recField = "_idlrec";
    }
    
    NewJavaFile(e->interface, shortname);

    if (e->doc_string != NULL) {
        printDocString("", e->doc_string);
    }
    printf("public class %s extends xerox.ilu.IluUserException {\n\n", 
          shortname);
    printf("    private static xerox.ilu.IluExceptionRep _%s_rep;\n", 
           shortname);
           
    printf("    public %s() {\n", shortname);
    printf("        super();\n");
    if (idlHack) {
        printf("        %s = new %s();\n", recField, typeDeclarator(e->type));
    }
    printf("    }\n\n");
        
    
    if (e->type) {
        char* declarator = typeDeclarator(e->type);
        if (idlHack) {
            printRecordFieldDecls(e->type, TRUE);
            printf("    private %s %s;\n\n", declarator, recField);
            printf("    public %s(", shortname);
            printRecordConstructorFormalArgs(e->type);
            printf(") {\n");
            printf("        super();\n");
            printf("        %s = new %s();\n", recField, declarator);
            printRecordAssignCorresponding(e->type, 
                cat3("        ", recField, "."), 
                "");
            printf("    }\n\n");
        } else {
            printf("    public %s %s;\n\n", declarator, recField);
            printf("    public %s(%s _val) {\n", 
                shortname, 
                declarator
                );
            printf("        super();\n");
            printf("        this.%s = _val;\n", recField);
            printf("    }\n\n");
        }
      
        
        printf("    //NOT really public; Must be accessible by ilu...\n");
        printf("    public void readException(xerox.ilu.IluCall _call)\n");
        printf("            throws xerox.ilu.IluSystemException {\n");
        printf("        this.%s = %s;\n", recField, ioInPiece(e->type));
        if (idlHack) {
            printRecordAssignCorresponding(e->type, 
                "        this.", 
                cat3("this.", recField, ".")
                );
        }
        printf("    } //readException\n\n");
        
        
        /* This is static because the dynamic exception might be 
         * a subclass which is not supported in the ISL definition
         */
        printf("    //NOT really public; Must be accessible by ilu...\n");
        printf("    public static void %s(%s _ex, int index, xerox.ilu.IluCall _call)\n",
            methodNameOfWriteProc(e),
            shortname
            );
        printf("            throws xerox.ilu.IluSystemException {\n");
        printf("        int sz = 0;\n");
        printf("        if (_call.needsSizing()) {\n");
        printf("            sz = _call.beginSizingException(index);\n");
        if (e->type) {
            printf("            sz += %s;\n", 
                ioSzPiece(e->type, cat2("_ex.", recField)));
        }
        printf("        }\n");
        printf("        _call.startWriteException(index, sz);\n");
        if (e->type) {
            printf("        %s;\n", 
                ioOutPiece(e->type, cat2("_ex.", recField)));
        }
        printf("        _call.doneWriteException();\n");
        printf("    } //%s\n\n", methodNameOfWriteProc(e));
    }
    
    printf("    public static final xerox.ilu.IluExceptionRep _%s_getRep() {\n", \
           shortname);
    printf("        return _%s_rep;\n", shortname);
    printf("    }\n\n");
    
    printf("    static {\n");
    printf("        _%s_rep = xerox.ilu.IluExceptionRep.defineException(\n", 
        shortname
        );
    printf("            \"%s\", //java class name\n",
        dotCat(packageName(currentIfc), shortname)
        );
    printf("            \"%s\", //isl interface name\n",
        interface_name(currentIfc)
        );
    printf("            \"%s\", //isl exception name\n", 
        exceptionShortName(e)
        );
    if (e->type) {
        printf("            \"%s\"); //type uuid\n", e->type->uid);
    } else {
        printf("            null); //type uuid\n");
    }
    
    printf("    }\n\n");
    printf("} //%s\n", shortname);
}


/* end */



